/*
 *   ScenarLang a domain specific language that attempts to make it even easier
 *   to write unit tests with mockito
 *    
 *   Copyright (C) 2012  X. Gillard 
 *   
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *   
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *   
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
package xgi.ut.dsl

import scala.util.parsing.combinator.JavaTokenParsers

/**
 * Defines a parser that treats comments as whitespace
 */
trait CommentsSkippingParser extends JavaTokenParsers {
  /* treats the comments as white space */
  override protected val whiteSpace = """(\s|//.*|(?m)/\*(\*(?!/)|[^*])*\*/)+""".r
  
  /* an utility method that builds a comprehensive error message */
  def errorString(error:NoSuccess):String = {
    """|Syntax error :
       |  -> Reason : %s
       |  -> At : %s 
       |%s""".stripMargin.format(error.msg, error.next.pos.toString, error.next.pos.longString)
  }
}